@@ -126,26 +126,6 @@ namespace Cpp {
 
 typedef QPair<Declaration*, int> DeclarationDepthPair;
 
-IndexedType switchExpressionType(DUContextPointer caseContext)
-{
-#ifndef TEST_COMPLETION
-  //Test thread has DUChain locked
-  ENSURE_CHAIN_NOT_LOCKED;
-#endif
-  ForegroundLock foregroundLock;
-  LOCKDUCHAIN;
-  if (!caseContext)
-    return IndexedType();
-  DUContext* switchContext = 0;
-  if (caseContext->importedParentContexts().size() == 1)
-    switchContext = caseContext->importedParentContexts().first().context(caseContext->topContext());
-  if (!switchContext)
-    return IndexedType();
-  QString switchExpression = switchContext->createRangeMoving()->text();
-  ExpressionParser expressionParser;
-  return expressionParser.evaluateType(switchExpression.toUtf8(), DUContextPointer(switchContext)).type;
-}
-
 //Search for a copy-constructor within class
 //*DUChain must be locked*
 bool hasCopyConstructor(CppClassType::Ptr classType, TopDUContext* topContext)
@@ -1318,11 +1298,15 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::memberAccessCompletionIt
       return items;
     ifDebug( kDebug() << "container:" << ctx->scopeIdentifier(true).toString(); )
 
-    foreach( const DeclarationDepthPair& decl, Cpp::hideOverloadedDeclarations( ctx->allDeclarations(ctx->range().end, m_duContext->topContext(), false ) ) ) {
+    foreach( const DeclarationDepthPair& decl,
+            Cpp::hideOverloadedDeclarations(
+              ctx->allDeclarations(ctx->range().end, m_duContext->topContext(), false ),
+              typeIsConst ) )
+    {
       //If we have StaticMemberChoose, which means A::Bla, show only static members, except if we're within a class that derives from the container
       ClassMemberDeclaration* classMember = dynamic_cast<ClassMemberDeclaration*>(decl.first);
 
-      if(classMember && !filterDeclaration(classMember, ctx, typeIsConst))
+      if(classMember && !filterDeclaration(classMember, ctx))
         continue;
       else if(!filterDeclaration(decl.first, ctx))
         continue;
@@ -1380,7 +1364,26 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::caseAccessCompletionItem
 {
   QList<CompletionTreeItemPointer> items;
 
-  IndexedType switchExprType = switchExpressionType(m_duContext);
+  {
+#ifndef TEST_COMPLETION
+    // Test thread has DUChain locked
+    // This is essential here - a ForegroundLock must be acquired _before_ locking DUChain
+    ENSURE_CHAIN_NOT_LOCKED;
+#endif
+
+    ForegroundLock foregroundLock;
+    LOCKDUCHAIN;
+
+    if( m_duContext && m_duContext->importedParentContexts().size() == 1 )
+    {
+      DUContext* switchContext = m_duContext->importedParentContexts().first().context( m_duContext->topContext() );
+      ExpressionParser expressionParser;
+      m_expression = switchContext->createRangeMoving()->text();
+      m_expressionResult = expressionParser.evaluateExpression( m_expression.toUtf8(), DUContextPointer( switchContext ) );
+    }
+  }
+
+  IndexedType switchExprType = m_expressionResult.type;
 
   LOCKDUCHAIN; if (!m_duContext) return items;
 
@@ -1676,12 +1679,12 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
         hadNamespaceDeclarations.insert(id);
       }
 
-      if(filterDeclaration(decl.first, 0, true, typeIsConst)) {
+      if(filterDeclaration(decl.first, 0, true)) {
         decls << decl;
       }
     }
     
-  decls = Cpp::hideOverloadedDeclarations(decls);
+  decls = Cpp::hideOverloadedDeclarations(decls, typeIsConst);
 
   foreach( const DeclarationDepthPair& decl, decls )
     items << CompletionTreeItemPointer( new NormalDeclarationCompletionItem(DeclarationPointer(decl.first), KDevelop::CodeCompletionContext::Ptr(this), decl.second ) );
@@ -1700,7 +1703,7 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
         }
       }
     }
-    else if (parent->accessType() == BinaryOpFunctionCallAccess)
+    else if (parent->accessType() == BinaryOpFunctionCallAccess || parent->accessType() == CaseAccess)
     {
       items += specialItemsForArgumentType(parent->m_expressionResult.type.abstractType());
     }
@@ -1731,7 +1734,7 @@ void CodeCompletionContext::addImplementationHelpers()
 {
   QList<CompletionTreeItemPointer> helpers = getImplementationHelpers();
   if(!helpers.isEmpty()) {
-    eventuallyAddGroup(i18n("Implement Function"), 0, helpers);
+    eventuallyAddGroup(i18nc("@action", "Implement Function"), 0, helpers);
   }
 }
 
@@ -1784,6 +1787,11 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
     if(!m_valid)
       return items;
 
+    // Call parent context before adding our items because if parent is CaseAccess, this call
+    // will make it compute its expression type (which we need in standardAccessCompletionItems())
+    if(shouldAddParentItems(fullCompletion))
+      items = parentContext()->completionItems( shouldAbort, fullCompletion );
+
     switch(m_accessType) {
       case MemberAccess:
       case ArrowMemberAccess:
@@ -1829,9 +1837,6 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
         break;
     }
 
-    if(shouldAddParentItems(fullCompletion))
-      items = parentContext()->completionItems( shouldAbort, fullCompletion ) + items;
-
     if(depth() == 0)
     {
       LOCKDUCHAIN; if (!m_duContext) return items;
@@ -1905,39 +1910,20 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpers
   return ret;
 }
 
-QualifiedIdentifier CodeCompletionContext::requiredPrefix(Declaration* decl) const {
-  QualifiedIdentifier worstCase = decl->context()->scopeIdentifier(true);
-  if(!m_duContext)
-    return worstCase;
-  QualifiedIdentifier currentPrefix;
-
-  while(1) {
-    QList<Declaration*> found = m_duContext->findDeclarations( currentPrefix + decl->identifier() );
-    if(found.contains(decl))
-      return currentPrefix;
-
-    if(currentPrefix.count() >= worstCase.count()) {
-      return worstCase;
-    }else {
-      currentPrefix.push(worstCase.at(currentPrefix.count()));
-    }
-  }
-}
-
 QList< KSharedPtr< KDevelop::CompletionTreeItem > > CodeCompletionContext::specialItemsForArgumentType(TypePtr< KDevelop::AbstractType > type) {
   QList< KSharedPtr< KDevelop::CompletionTreeItem > > items;
   if(EnumerationType::Ptr enumeration = TypeUtils::realType(type, m_duContext->topContext()).cast<EnumerationType>()) {
     Declaration* enumDecl = enumeration->declaration(m_duContext->topContext());
     if(enumDecl && enumDecl->internalContext()) {
 
-      QualifiedIdentifier prefix = requiredPrefix(enumDecl);
+      // This is necessary to avoid duplication of locally visible enumerators, which also get added by standardAccessCompletionItems().
+      if( visibleFromWithin( enumDecl, m_duContext.data() ) )
+        return items;
 
       DUContext* enumInternal = enumDecl->internalContext();
       foreach(Declaration* enumerator, enumInternal->localDeclarations()) {
-        QualifiedIdentifier id = prefix + enumerator->identifier();
         items << CompletionTreeItemPointer(new NormalDeclarationCompletionItem( DeclarationPointer(enumerator), KDevelop::CodeCompletionContext::Ptr(this), 0 ));
-        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->alternativeText = id.toString();
-        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->useAlternativeText = true;
+        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->prependScopePrefix = true;
       }
     }
   }
@@ -1969,7 +1955,7 @@ static inline int getIntegralReturnType(const AbstractType::Ptr& type)
   return intType->dataType();
 }
 
-bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* declarationContext, bool dynamic, bool typeIsConst) {
+bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* declarationContext, bool dynamic) {
   if(!decl)
     return true;
 
@@ -2023,7 +2009,7 @@ bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* dec
   if(dynamic && decl->context()->type() == DUContext::Class) {
     ClassMemberDeclaration* classMember = dynamic_cast<ClassMemberDeclaration*>(decl);
     if(classMember)
-      return filterDeclaration(classMember, declarationContext, typeIsConst);
+      return filterDeclaration(classMember, declarationContext);
   }
 
   // https://bugs.kde.org/show_bug.cgi?id=206376
@@ -2042,10 +2028,8 @@ bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* dec
   return true;
 }
 
-bool  CodeCompletionContext::filterDeclaration(ClassMemberDeclaration* decl, DUContext* declarationContext, bool typeIsConst) {
+bool  CodeCompletionContext::filterDeclaration(ClassMemberDeclaration* decl, DUContext* declarationContext) {
   if(m_doAccessFiltering && decl) {
-    if (typeIsConst && decl->type<FunctionType>() && !(decl->abstractType()->modifiers() & AbstractType::ConstModifier))
-      return false;
     if(!Cpp::isAccessible(m_localClass ? m_localClass.data() : m_duContext.data(), decl, m_duContext->topContext(), declarationContext))
       return false;
   }
