@@ -46,6 +46,41 @@ void TokenStream::splitRightShift(uint index)
   insert(index+1, next_token);
 }
 
+KDevelop::IndexedString TokenStream::symbol(const Token& t) const
+{
+  if(t.size == 1)
+    return KDevelop::IndexedString::fromIndex(session->contents()[t.position]);
+  else
+    return KDevelop::IndexedString();
+}
+
+uint TokenStream::symbolIndex(const Token& t) const
+{
+  return session->contents()[t.position];
+}
+
+QByteArray TokenStream::symbolByteArray(const Token& t) const
+{
+  if (t.size == 0) // esp. for EOF
+    return QByteArray();
+
+  return stringFromContents(session->contentsVector(), t.position, t.size);
+}
+
+QString TokenStream::symbolString(const Token& t) const
+{
+  return QString::fromUtf8(symbolByteArray(t));
+}
+
+uint TokenStream::symbolLength(const Token& t) const
+{
+  uint ret = 0;
+  for(uint a = t.position; a < t.position+t.size; ++a) {
+    ret += KDevelop::IndexedString::lengthFromIndex(session->contents()[a]);
+  }
+  return ret;
+}
+
 QString Lexer::SpecialCursor::toString() const
 {
   return KDevelop::IndexedString::fromIndex(*current).str();
@@ -118,40 +153,6 @@ void Lexer::skipComment()
   return;
 }
 
-KDevelop::IndexedString Token::symbol() const {
-  if(size == 1)
-    return KDevelop::IndexedString::fromIndex(session->contents()[position]);
-  else
-    return KDevelop::IndexedString();
-}
-
-uint Token::symbolIndex() const
-{
-  return session->contents()[position];
-}
-
-QByteArray Token::symbolByteArray() const {
-  if (size == 0) // esp. for EOF
-    return QByteArray();
-
-  return stringFromContents(session->contentsVector(), position, size);
-}
-
-QString Token::symbolString() const {
-  if (size == 0) // esp. for EOF
-    return QString();
-
-  return QString::fromUtf8(stringFromContents(session->contentsVector(), position, size));
-}
-
-uint Token::symbolLength() const {
-  uint ret = 0;
-  for(uint a = position; a < position+size; ++a) {
-    ret += KDevelop::IndexedString::lengthFromIndex(session->contents()[a]);
-  }
-  return ret;
-}
-
 const uint index_size = 200;
 
 KDevVarLengthArray<KDevVarLengthArray<QPair<uint, TOKEN_KIND>, 10 >, index_size > createIndicesForTokens() {
@@ -161,7 +162,6 @@ KDevVarLengthArray<KDevVarLengthArray<QPair<uint, TOKEN_KIND>, 10 >, index_size
   #define ADD_TOKEN2(string, tok) ret[KDevelop::IndexedString(#string).index() % index_size].append(qMakePair(KDevelop::IndexedString(#string).index(), Token_ ## tok));
   ADD_TOKEN(K_DCOP);
   ADD_TOKEN(Q_OBJECT);
-  ADD_TOKEN(__attribute__);
   ADD_TOKEN(__typeof);
   ADD_TOKEN2(__typeof__, __typeof);
   ADD_TOKEN2(typeof, __typeof);
@@ -206,6 +206,8 @@ KDevVarLengthArray<KDevVarLengthArray<QPair<uint, TOKEN_KIND>, 10 >, index_size
   ADD_TOKEN(goto);
   ADD_TOKEN(if);
   ADD_TOKEN(inline);
+  ADD_TOKEN2(__inline__, inline);
+  ADD_TOKEN2(__inline, inline);
   ADD_TOKEN(int);
   ADD_TOKEN(k_dcop);
   ADD_TOKEN(k_dcop_signals);
@@ -230,6 +232,7 @@ KDevVarLengthArray<KDevVarLengthArray<QPair<uint, TOKEN_KIND>, 10 >, index_size
   ADD_TOKEN(short);
   ADD_TOKEN(__qt_signals__);
   ADD_TOKEN(signed);
+  ADD_TOKEN2(__signed__, signed);
   ADD_TOKEN(sizeof);
   ADD_TOKEN(__qt_slots__);
   ADD_TOKEN(static);
@@ -250,6 +253,7 @@ KDevVarLengthArray<KDevVarLengthArray<QPair<uint, TOKEN_KIND>, 10 >, index_size
   ADD_TOKEN(typename);
   ADD_TOKEN(union);
   ADD_TOKEN(unsigned);
+  ADD_TOKEN2(__unsigned__, unsigned);
   ADD_TOKEN(using);
   ADD_TOKEN(virtual);
   ADD_TOKEN(void);
@@ -291,7 +295,6 @@ void Lexer::tokenize(ParseSession* _session)
   {
   Token eof;
   eof.kind = Token_EOF;
-  eof.session = session;
   eof.position = 0;
   eof.size = 0;
   stream->append(eof);
@@ -310,7 +313,6 @@ void Lexer::tokenize(ParseSession* _session)
 
     {
     Token token;
-    token.session = session;
     token.position = cursor.offsetIn( session->contents() );
     token.size = 0;
     stream->append(token);
@@ -368,7 +370,6 @@ void Lexer::tokenize(ParseSession* _session)
   {
   Token eof;
   eof.kind = Token_EOF;
-  eof.session = session;
   eof.position = cursor.offsetIn( session->contents() );
   eof.size = 0;
   stream->append(eof);
@@ -896,7 +897,6 @@ void Lexer::scan_divide()
           (*session->token_stream)[index++].kind = Token_comment;
           (*session->token_stream)[index-1].size = (size_t)(cursor - commentBegin);
           (*session->token_stream)[index-1].position = commentBegin.offsetIn( session->contents() );
-          (*session->token_stream)[index-1].session = session;
         }else{
           //Merge with previous comment
           (*session->token_stream)[index-1].size = cursor.offsetIn(session->contents()) - (*session->token_stream)[index-1].position;
